

import at.jku.ssw.BubbleSort;
import at.jku.ssw.InsertionSort;
import at.jku.ssw.JavaUtilArraysSort;
import at.jku.ssw.Measurement;
import at.jku.ssw.Out;
import at.jku.ssw.QuickSort;
import at.jku.ssw.SelectionSort;
import at.jku.ssw.Sorter;
import at.jku.ssw.SorterEvaluationUI;
import at.jku.ssw.Timer;
import java.util.Arrays;

import at.jku.students.HeapSort;
import at.jku.students.MergeSort;

public class SortTest {

  public static void main(String[] args) {
    test1();
    test2();
    test3();
  }

  static void test1() {
    Sorter heapSort = new HeapSort();
    Comparable[] values = new Integer[]{10, 3, 1, 7, 6, 5, 4, 2, 9, 8};
    values = heapSort.sort(values);
    for (int i = 0; i < values.length; i++) {
      Out.print(" " + values[i]);
    }
    Out.println();
  }

  static void test2() {
    Sorter heapSort = new MergeSort();
    Comparable[] values = new Integer[]{10, 3, 1, 7, 6, 5, 4, 2, 9, 8};
    values = heapSort.sort(values);
    for (int i = 0; i < values.length; i++) {
      Out.print(" " + values[i]);
    }
    Out.println();
  }

  static void test3() {
    Comparable[] data = generateIntegers(10000, Order.RANDOM);

    Sorter[] sorters = new Sorter[]{
      new BubbleSort(), new SelectionSort(),
      new InsertionSort(), new MergeSort(),
      new JavaUtilArraysSort(), new QuickSort(),
      new HeapSort()};

    Measurement[] measurements = new Measurement[sorters.length];
    for (int i = 0; i < sorters.length; i++) {
      measurements[i] = new Measurement(sorters[i], data);
    }

    final int WARMUP = 10;
    for (int i = 0; i < sorters.length; i++) {
      repeatMeasurements(measurements[i], WARMUP);
    }

    final int REPEAT = 10;
    for (int i = 0; i < measurements.length; i++) {
      repeatMeasurements(measurements[i], REPEAT);
    }
    Arrays.sort(measurements);

    new SorterEvaluationUI(measurements, data).show();
  }

  private static void repeatMeasurements(Measurement measurement, int repeat) {
    measurement.timer = Timer.start();
    for (int i = 0; i < repeat; i++) {
      measurement.timer.pause();
      Comparable[] data = measurement.unsortedData.clone();
      measurement.timer.continu();
      measurement.sortedData = measurement.sorter.sort(data);
    }
    measurement.timer.stop();
  }

  public static enum Order {

    ASCENDING, DESCENDING, RANDOM
  };

  private static Comparable[] generateIntegers(int cnt, Order order) {
    Integer[] arr = new Integer[cnt];
    for (int i = 0; i < cnt; i++) {
      switch (order) {
        case ASCENDING:
          arr[i] = i;
          break;
        case DESCENDING:
          arr[i] = cnt - i;
          break;
        case RANDOM:
          arr[i] = (int) (Math.random() * cnt);
          break;
      }
    }
    return arr;
  }

}
